exec("./Support_LibStr.cs");
exec("./Support_UpdaterDownload.cs");
exec("./options.cs");

if($Pref::Themes::DefaultPrefVersion !$= "0.1.0")
{
	$Pref::Themes::DefaultPrefVersion = "0.1.0";
	$Pref::Themes::CurrentTheme = "";
}

$Themes::CurrentTheme = $Pref::Themes::CurrentTheme;
$Themes::DefaultThemePath = "base/client/ui";

//Applies a theme to all GUIs that use the default theme. Set %themePath to "" to restore default theme.
//@param	string themePath	The location where theme files can be found. e.g. "Add-Ons/Theme_Example"
function Themes::applyTheme(%this, %themePath)
{
	if(%themePath $= "Default")
		%themePath = "";
	%restore = (%themePath $= "");
	if(%restore)
	{
		echo("Restoring default theme.");
	}
	else
	{
		if(!isFile(%themePath @ "/description.txt"))
		{
			error("ERROR: Theme \"" @ %themePath @ "\" does not contain a description.txt file.");
			return;
		}
		//restore default theme first
		if(%this.themePath !$= "")
		{
			%this.applyTheme("");
			//restart required flag
			if(isFile(%this.themePath @ "/tweaks.cs"))
			{
				%this.restartRequired = true;
			}
		}
		echo("Applying theme:" SPC %themePath);
	}
	%this.scheduleNoQuota(0, "applyThemeStage2", %themePath, %restore);
}

//Handles the actual theme change.
//@param	string themePath
//@param	bool restore
//@see	Themes::applyTheme
//@PRIVATE
function Themes::applyThemeStage2(%this, %themePath, %restore)
{
	%this.setCanvasReady(true);
	%this.themeApplying = true;

	if(%restore)
	{
		%this.restoreGuiTree(GuiGroup);
		%this.restoreCursor();
		%this.themePath = "";
		%this.themeName = "";
	}
	else
	{
		%this.themePath = %themePath;
		%this.themeName = stripChars(%themePath, "~`!@#$%^&*()-=+[{]}\\|;:\'\",<.>? ");
		%this.themeName = getField(strReplace(%this.themeName, "/", "\t"), 1);

		%profiles = %themePath @ "/profiles.cs";
		if(isFile(%profiles))
			exec(%profiles);

		%this.themeGuiTree(GuiGroup, %themePath);
		%this.themeCursor(%themePath);

		%tweaks = %themePath @ "/tweaks.cs";
		if(isFile(%tweaks))
			exec(%tweaks);
	}

	%this.themedOnce = true;
	%this.themeApplying = false;
	%this.setCanvasReady(false);

	if(%this.restartRequired)
	{
		messageBoxYesNo(
			"Restart Required",
			"The theme has been applied, but a restart may be required before all elements display properly." @
				"\n\n<font:Arial Bold:14>Close Blockland now?",
			"quit();");
	}
}

//Sets whether the canvas is ready to have a theme applied to it.
//@param	bool canvasReady
function Themes::setCanvasReady(%this, %canvasReady)
{
	if(%canvasReady && !%this.canvasReady)
	{
		%this.canvasReady = true;
		ThemeBlankMessage.setVisible(%this.themedOnce ? true : false);
		%this.restoreContent = canvas.getObject(0);
		canvas.setContent(ThemeBlank);
		canvas.repaint();
	}
	else if(!%canvasReady && %this.canvasReady)
	{
		%this.canvasReady = false;
		if(!isObject(%this.restoreContent))
			canvas.setContent(MainMenuGui);
		canvas.setContent(%this.restoreContent);
	}
}

//Applies a theme to the entire tree of %gui's child objects.
//@param	SimSet gui
//@param	string themePath
function Themes::themeGuiTree(%this, %gui, %themePath)
{
	for(%i = 0; %i < %gui.getCount(); %i ++)
	{
		%child = %gui.getObject(%i);
		if(!%child.themeIgnore)
		{
			%this.themeGui(%child, %themePath);
			if(%child.getCount() > 0)
			{
				%this.themeGuiTree(%child, %themePath);
			}
		}
	}
}

//Applies a theme to a specific GUI object.
//@param	GuiControl gui
//@param	string themePath
function Themes::themeGui(%this, %gui, %themePath)
{
	%profile = %gui.profile;
	if(!isObject(%profile))
		return;
	%themeProfileName = %this.themeName @ "_" @ %profile.getName();
	%themeProfile = nameToID(%themeProfileName);
	if(!isObject(%themeProfile))
	{
		%bitmap = %profile.bitmap;
		%themeBitmap = %this.getThemeBitmap(%themePath, %bitmap);
		if(%themeBitmap $= "")
		{
			if(%gui.themed)
				%this.restoreGui(%gui);
			return;
		}
		else
		{
			%profileName = %profile.getName();
			%profile.setName("ThemeParentProfile");
			%themeProfile = new GuiControlProfile(ThemeNewProfile : ThemeParentProfile)
			{
				bitmap = %themeBitmap;
				parent = %profile;
				themePath = %themePath;
			};
			ThemeData.add(ThemeNewProfile);
			ThemeNewProfile.setName(%themeProfileName);
			ThemeParentProfile.setName(%profileName);
		}
	}

	%gui.themeApplying = true;

	%gui.setProfile(%themeProfile);

	if(!%this.themeApplying)
	{
		cancel(canvas.repaintSchedule);
		canvas.repaintSchedule = canvas.schedule(0, "repaint");
	}

	if(%gui.bitmap !$= "" && isFunction(%gui.getClassName(), "setBitmap"))
	{
		%bitmap = %gui.bitmap;
		if(%gui.themed || striPos(%bitmap, $Themes::DefaultThemePath) == 0)
		{
			%themeBitmap = %this.getThemeBitmap(%themePath, %bitmap);
			if(%themeBitmap !$= "")
			{
				%gui.setBitmap(%themeBitmap);
				%gui.themedBitmap = true;
			}
		}
	}

	%gui.themeApplying = false;

	%gui.themed = true;
}

//Sets the cursor to the theme's custom cursor.
//@param	string themePath
function Themes::themeCursor(%this, %themePath)
{
	%cursorFile = %themePath @ "/CUR_3darrow.png";
	if(isFile(%cursorFile))
	{
		%name = %this.themeName @ "_Cursor";
		%cursor = nameToID(%name);
		if(!isObject(%cursor))
		{
			%cursor = new GuiCursor(%name)
			{
				bitmapName = %cursorFile;
				hotspot = "0 0";
			};
		}
		canvas.setCursor(%cursor);
	}
}

//Restores the cursor to its default state.
function Themes::restoreCursor(%this)
{
	if(isObject(DefaultCursor))
		canvas.setCursor(DefaultCursor);
}

//Restores an entire tree of GUI objects to its default state.
//@param	SimSet gui
function Themes::restoreGuiTree(%this, %gui)
{
	for(%i = 0; %i < %gui.getCount(); %i ++)
	{
		%child = %gui.getObject(%i);
		if(!%child.themeIgnore)
		{
			%this.restoreGui(%child);
			if(%child.getCount() > 0)
			{
				%this.restoreGuiTree(%child);
			}
		}
	}
}

//Restores a specific GUI to its default state.
//@param	GuiControl gui
function Themes::restoreGui(%this, %gui)
{
	if(!%gui.themed)
		return;
	%gui.themed = "";
	%profile = %gui.profile;
	%name = %profile.getName();
	if(striPos(%name, %this.themeName) == 0)
	{
		%parentName = striReplace(%name, %this.themeName @ "_", "");
		%parent = nameToID(%parentName);
		if(isObject(%parent))
		{
			%gui.setProfile(%parent);
		}
	}
	if(%gui.themedBitmap && isFunction(%gui.getClassName(), "setBitmap"))
	{
		%bitmap = setSubStr(%gui.bitmap, 0, strLen(%this.themePath), $Themes::DefaultThemePath);
		%gui.setBitmap(%bitmap);
		%gui.themedBitmap = "";
	}
}

//Given a default bitmap, returns the theme's replacement image.
//@param	string themePath
//@param	string bitmap
//@return	string	Returns the image path if found, "" if no image found.
function Themes::getThemeBitmap(%this, %themePath, %bitmap)
{
	//replace default path with theme path
	%themeBitmap = setSubStr(%bitmap, 0, strLen($Themes::DefaultThemePath), %themePath);
	//check if file exists
	if(%this.isThemeBitmap[%themeBitmap] $= "")
	{
		%this.isThemeBitmap[%themeBitmap] =
			isFile(%themeBitmap) ||
			isFile(%themeBitmap @ ".png") ||
			isFile(%themeBitmap @ ".jpg") ||
			isFile(findFirstFile(%themeBitmap @ "_*"));
	}
	if(!%this.isThemeBitmap[%themeBitmap])
		return "";
	return %themeBitmap;
}

//Checks whether a GUI should not be themed.
//@param	GuiControl gui
//@return	bool
function Themes_themeIgnore(%gui)
{
	if(%gui.themeIgnore)
		return true;
	else if(isObject(%parent = %gui.getGroup()))
		return Themes_themeIgnore(%parent);
	else
		return false;
}

//Called when a bitmap is changed.
//@param	GuiControl gui
//@param	string bitmap
function Themes_onSetBitmap(%gui, %bitmap)
{
	%gui.bitmap = %bitmap;
	if(%gui.themedBitmap && Themes.themePath !$= "" &&
		striPos(%bitmap, $Themes::DefaultThemePath) == 0 &&
		!Themes.themeApplying && !Themes_themeIgnore(%gui))
	{
		%themeBitmap = Themes.getThemeBitmap(Themes.themePath, %bitmap);
		if(%themeBitmap !$= "")
		{
			%gui.setBitmap(%themeBitmap);
		}
	}
}

//create a dummy function so that I can parent it properly
if(!isFunction("GuiControl", "onAdd"))
	eval("function GuiControl::onAdd(%this){}");

package Themes
{
	//@private
	function cursorOn()
	{
		parent::cursorOn();
		if(Themes.themePath !$= "")
			Themes.themeCursor(Themes.themePath);
	}

	//@private
	function GuiControl::onAdd(%this)
	{
		parent::onAdd(%this);
		if(Themes.themePath !$= "" && !Themes_themeIgnore(%this))
		{
			Themes.schedule(0, "themeGui", %this, Themes.themePath);
		}
	}

	//@private
	function GuiAnimatedBitmapCtrl::setBitmap(%this, %bitmap)
	{
		parent::setBitmap(%this, %bitmap);
		Themes_onSetBitmap(%this, %bitmap);
	}

	//@private
	function GuiBitmapButtonCtrl::setBitmap(%this, %bitmap)
	{
		parent::setBitmap(%this, %bitmap);
		Themes_onSetBitmap(%this, %bitmap);
	}

	//@private
	function GuiBitmapCtrl::setBitmap(%this, %bitmap)
	{
		parent::setBitmap(%this, %bitmap);
		Themes_onSetBitmap(%this, %bitmap);
	}

	//@private
	function GuiControlProfile::updateFont(%this)
	{
		parent::updateFont(%this);
		if(Themes.themePath !$= "" && !Themes_themeIgnore(%this))
		{
			%name = Themes.themeName @ "_" @ %this.getName();
			%profile = nameToID(%name);
			if(isObject(%profile))
			{
				%profile.fontSize = %this.fontSize;
				%profile.updateFont();
			}
		}
	}
};
activatePackage(Themes);

if(!isObject(Themes))
{
	new ScriptGroup(Themes)
	{
		new SimSet(ThemeData);
		new GuiControl(ThemeBlank)
		{
			profile = GuiDefaultProfile;
			position = "0 0";
			extent = "800 600";
			enabled = "1";
			visible = "1";
			themeIgnore = true;

			new GuiSwatchCtrl()
			{
				profile = "GuiDefaultProfile";
				horizSizing = "width";
				vertSizing = "height";
				position = "0 0";
				extent = "800 600";
				minExtent = "8 2";
				enabled = "1";
				visible = "1";
				clipToParent = "1";
				color = "0 0 0 255";

				new GuiMLTextCtrl(ThemeBlankMessage) {
					profile = "GuiMLTextProfile";
					horizSizing = "center";
					vertSizing = "center";
					position = "206 228";
					extent = "228 24";
					minExtent = "8 2";
					enabled = "1";
					visible = "1";
					clipToParent = "1";
					lineSpacing = "2";
					allowColorChars = "0";
					maxChars = "-1";
					text = "<just:center><color:ffffff><font:Arial Bold:24>Applying theme...";
					maxBitmapHeight = "-1";
					selectable = "0";
					autoResize = "1";
				};
			};
		};
	};
	Themes.schedule(0, "applyTheme", $Pref::Themes::CurrentTheme);
}